<?php

/* 	
**	This file deals with the creation of the Order Custom Profile Fields order column.
**	It also has to deal with old or inconsistant data in the order column of each profile field.
*/

// Handle running this file by using SSI.php
if (file_exists(dirname(__FILE__) . '/SSI.php') && !defined('SMF'))
	require_once(dirname(__FILE__) . '/SSI.php');
elseif (!defined('SMF'))
	die ('<b>Error:</b> Cannot install - please verify you put this in the same place as SMF\'s index.php.');

// Make sure that we have the package database functions.
if (!array_key_exists('db_add_column', $smcFunc))
	db_extend('packages');

// We need to work out, if the 'position' column already exists, assume that it doesn't.
$column_exsists = false;

// Go through a list of columns from the 'custom_fields' to check for the 'position' colunm.
foreach ($smcFunc['db_list_columns']('{db_prefix}custom_fields') as $column)
	if ($column == 'position')
		$column_exsists = true;

// Set a starting position value, it will be needed later on.
$current_pos = 0;

// This variable determines whether or not we need to completely resort the fields.
$clean_sort = false;

// So, does the colunm already exist? If it doesn't then we need to create it.
if (empty($column_exsists))
{
	// Setup the column information.
	$column_info = array(
		'name' => 'position',
		'type' => 'smallint',
		'size' => '5',
		// Just as a note to the Customization Team. This is actually totally unecessary, because the smf_db_add_column() function ignores empty default values. :P
		'default' => 0,
	);
	// Create the column itself.
	$smcFunc['db_add_column']('{db_prefix}custom_fields', $column_info);

	// We will need to sort the fields.
	$clean_sort = true;
}
// Otherwise, we have to assume that there is already some data and we have to do make sure that it is all in the correct order.
else
{
	// Firstly lets get a list of current profile fields.
	$request = $smcFunc['db_query']('','
		SELECT position, id_field
		FROM {db_prefix}custom_fields
		ORDER BY position',
		array()
	);

	// Make an array of current profile fields.
	$fields = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$fields[$row['id_field']] = $row['position'];

	// Free the db request.
	$smcFunc['db_free_result']($request);

	// We might need to upgrade the fields positioning, previously field positions started at 1 not 0.
	$upgrade = false;

	// Check to see if there are any positions with multiple fields, in that case, we'll just do a clean sort.
	foreach ($fields as $id => $position)
	{
		if (!$upgrade)
		{
			if ($position == 1 && $current_pos + 1 == $position)
				$upgrade = true;
			elseif ($current_pos != $position)
			{
				$clean_sort = true;
				break;
			}
			$current_pos++;
		}
		else
		{
			if (++$current_pos != $position)
			{
				$clean_sort = true;
				break;
			}
		}
	}

	// If we don't have to do a clean sort, then we may be upgrading.
	if ($upgrade && !$clean_sort)
	{
		$current_pos = 0;

		// Give each field a valid position value.
		foreach ($fields as $field_id)
			$smcFunc['db_query']('','
				UPDATE {db_prefix}custom_fields
				SET position = {int:position}
				WHERE id_field = {int:field_id}',
				array(
					'position' => $current_pos++,
					'field_id' => $field_id,
				)
			);
	}
}

// To ensure valid data, trash any old postion data and setup positions to sort by id.
if ($clean_sort)
{
	$current_pos = 0;

	// Get a list of fields, sorted by their id. This is the sorting that SMF uses by default.
	$request = $smcFunc['db_query']('','
		SELECT id_field
		FROM {db_prefix}custom_fields	
		ORDER BY id_field',
		array()
	);

	// Make an array of the profile fields.
	$fields = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$fields[] = $row['id_field'];

	// Free the db request.
	$smcFunc['db_free_result']($request);

	// Give each field a valid position value.
	foreach ($fields as $field_id)
		$smcFunc['db_query']('','
			UPDATE {db_prefix}custom_fields
			SET position = {int:position}
			WHERE id_field = {int:field_id}',
			array(
				'position' => $current_pos++,
				'field_id' => $field_id,
			)
		);
}

//Save the highest position as a setting in the database.
updateSettings(array('custom_profile_fields_pos_count' => $current_pos - 1));

// Make sure that the user has the option to remove the column when they uninstall the mod.
$db_package_log = array(array('remove_column', '{db_prefix}custom_fields', 'position'));

// Show a confirmation message if an upgrade through SSI.php was successful.
if (SMF == 'SSI')
   echo 'Database Updates Complete!';

?>